



if not exist %LIBRARY_LIB%\\yasm.lib exit 1
IF %ERRORLEVEL% NEQ 0 exit 1
if not exist %LIBRARY_BIN%\\yasm.dll exit 1
IF %ERRORLEVEL% NEQ 0 exit 1
vsyasm --help
IF %ERRORLEVEL% NEQ 0 exit 1
yasm --help
IF %ERRORLEVEL% NEQ 0 exit 1
ytasm /h
IF %ERRORLEVEL% NEQ 0 exit 1
yasm --version
IF %ERRORLEVEL% NEQ 0 exit 1
exit 0
